@@ -36,8 +36,9 @@
 #include <errno.h>
 #include "db.h"
 #include <sys/stat.h>
+#include <sys/time.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <libedataserver/e-dbhash.h>
 #include <libedataserver/e-db3-utils.h>
 #include <libedataserver/e-util.h>
@@ -70,6 +71,12 @@ struct _EBookBackendFilePrivate {
 	EBookBackendSummary *summary;
 };
 
+static GStaticMutex global_env_lock = G_STATIC_MUTEX_INIT;
+static struct {
+	int ref_count;
+	DB_ENV *env;
+} global_env;
+
 static void
 string_to_dbt(const char *str, DBT *dbt)
 {
@@ -121,6 +128,8 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 
 	}
+
+	dbc->c_close (dbc);
 }
 
 static char *
@@ -133,6 +142,21 @@ e_book_backend_file_create_unique_id (void)
 	return g_strdup_printf (PAS_ID_PREFIX "%08lX%08X", time(NULL), c++);
 }
 
+static void 
+set_revision (EContact *contact)
+{
+	char time_string[25] = {0};
+	const struct tm *tm = NULL;
+	struct timeval tv;
+
+	if (!gettimeofday (&tv, NULL))
+		tm = gmtime (&tv.tv_sec);
+	if (tm)
+		strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	e_contact_set (contact, E_CONTACT_REV, time_string);
+	
+}
+
 static EContact *
 do_create(EBookBackendFile  *bf,
 	  const char      *vcard_req)
@@ -143,13 +167,18 @@ do_create(EBookBackendFile  *bf,
 	char           *id;
 	EContact       *contact;
 	char           *vcard;
-
+	const char *rev;
+	
 	id = e_book_backend_file_create_unique_id ();
 
 	string_to_dbt (id, &id_dbt);
 
 	contact = e_contact_new_from_vcard (vcard_req);
-	e_contact_set(contact, E_CONTACT_UID, id);
+	e_contact_set (contact, E_CONTACT_UID, id);
+	rev = e_contact_get_const (contact,  E_CONTACT_REV);
+	if (!(rev && *rev))
+		set_revision (contact);
+
 	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 
 	string_to_dbt (vcard, &vcard_dbt);
@@ -253,6 +282,7 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 	DBT            id_dbt, vcard_dbt;
 	int            db_error;
 	char          *id, *lookup_id;
+	char          *vcard_with_rev;
 
 	*contact = e_contact_new_from_vcard (vcard);
 	id = e_contact_get(*contact, E_CONTACT_UID);
@@ -282,7 +312,11 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 
 	free (vcard_dbt.data);
 
-	string_to_dbt (vcard, &vcard_dbt);	
+	/* update the revisio (modified time of contact) */
+	set_revision (*contact);
+	vcard_with_rev = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
+	
+	string_to_dbt (vcard_with_rev, &vcard_dbt);	
 
 	db_error = db->put (db, NULL, &id_dbt, &vcard_dbt, 0);
 
@@ -295,6 +329,7 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
 	}
 	g_free (id);
+	g_free (vcard_with_rev);
 
 	if (0 == db_error)
 		return GNOME_Evolution_Addressbook_Success;
@@ -325,6 +360,7 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 
 	if (db_error == 0) {
 		*vcard = g_strdup (vcard_dbt.data);
+		printf ("backend %s\n", *vcard);
 		free (vcard_dbt.data);
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
@@ -349,50 +385,80 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 	gboolean search_needed;
 	const char *search = query;
 	GList *contact_list = NULL;
-
+	EBookBackendSyncStatus status;
+	
 	d(printf ("e_book_backend_file_get_contact_list (%s)\n", search));
+	status = GNOME_Evolution_Addressbook_Success;
+	if (e_book_backend_summary_is_summary_query (bf->priv->summary, search)) {
+	
+		/* do a summary query */
+		GPtrArray *ids = e_book_backend_summary_search (bf->priv->summary, search);
+		int i;
 
-	search_needed = TRUE;
+		for (i = 0; i < ids->len; i ++) {
+			char *id = g_ptr_array_index (ids, i);
+			string_to_dbt (id, &id_dbt);
+			memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+			vcard_dbt.flags = DB_DBT_MALLOC;
 
-	if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
-		search_needed = FALSE;
-
-	card_sexp = e_book_backend_sexp_new (search);
-	if (!card_sexp) {
-		/* XXX this needs to be an invalid query error of some sort*/
-		return GNOME_Evolution_Addressbook_ContactNotFound;
-	}
-
-	db_error = db->cursor (db, NULL, &dbc, 0);
-
-	if (db_error != 0) {
-		/* XXX this needs to be some CouldNotOpen error */
-		return GNOME_Evolution_Addressbook_ContactNotFound;
-	}
-
-	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
-	memset (&id_dbt, 0, sizeof (id_dbt));
-	db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
-
-	while (db_error == 0) {
-
-		/* don't include the version in the list of cards */
-		if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
-		    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-
-			if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
+			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
+			if (db_error == 0)
 				contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+			else { 
+				status = GNOME_Evolution_Addressbook_OtherError ;
+				break;
 			}
 		}
+		g_ptr_array_free (ids, TRUE);
+		
+	} else {
+		search_needed = TRUE;
+		if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
+			search_needed = FALSE;
 
-		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+		card_sexp = e_book_backend_sexp_new (search);
+		if (!card_sexp) {
+			/* XXX this needs to be an invalid query error of some sort*/
+			return GNOME_Evolution_Addressbook_ContactNotFound;
+		}
 
+		db_error = db->cursor (db, NULL, &dbc, 0);
+
+		if (db_error != 0) {
+			/* XXX this needs to be some CouldNotOpen error */
+			return GNOME_Evolution_Addressbook_ContactNotFound;
+		}
+
+		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+		memset (&id_dbt, 0, sizeof (id_dbt));
+		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
+
+		while (db_error == 0) {
+
+			/* don't include the version in the list of cards */
+			if (id_dbt.size != strlen(E_BOOK_BACKEND_FILE_VERSION_NAME) + 1
+			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
+
+				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
+					contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+				}
+			}
+
+			db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
+
+		}
+		status = db_error != DB_NOTFOUND
+			? GNOME_Evolution_Addressbook_OtherError
+			: GNOME_Evolution_Addressbook_Success;
+
+		db_error = dbc->c_close(dbc);
+		if (db_error != 0) {
+			g_warning("Could not close cursor: %d", db_error);
+		}
 	}
 
 	*contacts = contact_list;
-	return db_error != DB_NOTFOUND
-		? GNOME_Evolution_Addressbook_OtherError
-		: GNOME_Evolution_Addressbook_Success;
+	return status;
 }
 
 typedef struct {
@@ -473,7 +539,7 @@ book_view_thread (gpointer data)
 
 		for (i = 0; i < ids->len; i ++) {
 			char *id = g_ptr_array_index (ids, i);
-
+			printf ("**** id %s\n", id);
 			g_mutex_lock (closure->mutex);
 			stopped = closure->stopped;
 			g_mutex_unlock (closure->mutex);
@@ -491,6 +557,7 @@ book_view_thread (gpointer data)
 				EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
 				/* notify_update will check if it matches for us */
 				e_data_book_view_notify_update (book_view, contact);
+				
 				g_object_unref (contact);
 
 				free (vcard_dbt.data);
@@ -792,8 +859,7 @@ e_book_backend_file_get_required_fields (EBookBackendSync *backend,
 					  GList **fields_out)
 {
 	GList *fields = NULL;
-	int i;
-
+	
 	fields = g_list_append (fields , g_strdup(e_contact_field_name (E_CONTACT_FILE_AS)));
 	*fields_out = fields;
 	return GNOME_Evolution_Addressbook_Success;
@@ -984,14 +1050,34 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
-	db_error = db_env_create (&env, 0);
-	if (db_error != 0) {
-		g_warning ("db_env_create failed with %d", db_error);
-		return GNOME_Evolution_Addressbook_OtherError;
-	}
-	env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
+	g_static_mutex_lock(&global_env_lock);
+	if (global_env.ref_count > 0) {
+		env = global_env.env;
+		global_env.ref_count++;
+	} else {
+		db_error = db_env_create (&env, 0);
+		if (db_error != 0) {
+			g_warning ("db_env_create failed with %d", db_error);
+			g_static_mutex_unlock(&global_env_lock);
+			return GNOME_Evolution_Addressbook_OtherError;
+		}
 
-	env->set_errcall (env, file_errcall);
+		db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
+		if (db_error != 0) {
+			env->close(env, 0);
+			g_warning ("db_env_open failed with %d", db_error);
+			g_static_mutex_unlock(&global_env_lock);
+			return GNOME_Evolution_Addressbook_OtherError;
+		}
+
+		env->set_errcall (env, file_errcall);
+
+		global_env.env = env;
+		global_env.ref_count = 1;
+	}
+	g_static_mutex_unlock(&global_env_lock);
+
+	bf->priv->env = env;
 
 	db_error = db_create (&db, env, 0);
 	if (db_error != 0) {
@@ -1218,8 +1304,13 @@ e_book_backend_file_dispose (GObject *object)
 		if (bf->priv->file_db)
 			bf->priv->file_db->close (bf->priv->file_db, 0);
 
-		if (bf->priv->env)
-			bf->priv->env->close (bf->priv->env, 0);
+		g_static_mutex_lock(&global_env_lock);
+		global_env.ref_count--;
+		if (global_env.ref_count == 0) {
+			global_env.env->close(global_env.env, 0);
+			global_env.env = NULL;
+		}
+		g_static_mutex_unlock(&global_env_lock);
 
 		if (bf->priv->summary)
 			g_object_unref(bf->priv->summary);
