@@ -15,7 +15,7 @@
 #include <bonobo/bonobo-main.h>
 #include <gconf/gconf-client.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <libedataserver/e-component-listener.h>
 
 #include "e-book-marshal.h"
@@ -401,12 +401,11 @@ e_book_response_add_contact (EBook       *book,
 	}
 
 	op->id = g_strdup (id);
+	op->status = status;
 
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
-
 		g_cond_signal (op->cond);
 
 		g_mutex_unlock (op->mutex);
@@ -848,10 +847,10 @@ e_book_response_get_required_fields (EBook       *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
 		op->list = fields;
 
 		g_cond_signal (op->cond);
@@ -898,10 +897,10 @@ e_book_response_get_supported_fields (EBook       *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
 		op->list = fields;
 
 		g_cond_signal (op->cond);
@@ -1077,10 +1076,10 @@ e_book_response_get_supported_auth_methods (EBook                 *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
 		op->list = auth_methods;
 
 		g_cond_signal (op->cond);
@@ -2073,19 +2072,17 @@ e_book_response_get_contacts (EBook       *book,
 
 	op->list = g_list_copy (contact_list);
 	g_list_foreach (op->list, (GFunc)g_object_ref, NULL);
+	op->status = status;
 
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
-
 		g_cond_signal (op->cond);
 
 		g_mutex_unlock (op->mutex);
 	}
 	else {
 		op->book = g_object_ref (book);
-		op->status = status;
 		op->idle_id = g_idle_add (emit_async_get_contacts_response, op);
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
@@ -2334,18 +2331,16 @@ e_book_response_generic (EBook       *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
-
 		g_cond_signal (op->cond);
 
 		g_mutex_unlock (op->mutex);
 	}
 	else {
 		op->book = g_object_ref (book);
-		op->status = status;
 		op->idle_id = g_idle_add (emit_async_generic_response, op);
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
@@ -2387,7 +2382,7 @@ e_book_cancel (EBook   *book,
 	if (e_book_get_current_sync_op (book) == NULL) {
 		g_mutex_unlock (book->priv->mutex);
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_COULD_NOT_CANCEL,
-			     _("e_book_cancel: there is no current operation"));
+			     _("%s: there is no current operation"), "e_book_cacnel");
 		return FALSE;
 	}
 
@@ -2422,7 +2417,7 @@ e_book_cancel (EBook   *book,
 	}
 	else {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_COULD_NOT_CANCEL,
-			     _("e_book_cancel: couldn't cancel"));
+			     _("%s: couldn't cancel"), "e_book_cancel");
 		rv = FALSE;
 	}
 
@@ -2521,7 +2516,7 @@ do_open (gboolean sync,
 			/* Cancelled */
 			book->priv->load_state = E_BOOK_SOURCE_NOT_LOADED;
 			g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_CANCELLED,
-				     _("e_book_open: cancelled"));
+				     _("%s: cancelled"), "e_book_open");
 			return FALSE;
 		}
 		else if (status == E_BOOK_ERROR_OK) {
@@ -2624,18 +2619,16 @@ e_book_response_open (EBook       *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
-
 		g_cond_signal (op->cond);
 
 		g_mutex_unlock (op->mutex);
 	}
 	else {
 		op->book = g_object_ref (book);
-		op->status = status;
 		op->idle_id = g_idle_add (emit_async_open_response, op);
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
@@ -2774,18 +2767,16 @@ e_book_response_remove (EBook       *book,
 		return;
 	}
 
+	op->status = status;
 	if (op->synchronous) {
 		g_mutex_lock (op->mutex);
 
-		op->status = status;
-
 		g_cond_signal (op->cond);
 
 		g_mutex_unlock (op->mutex);
 	}
 	else {
 		op->book = g_object_ref (book);
-		op->status = status;
 		op->idle_id = g_idle_add (emit_async_generic_response, op);
 		book->priv->pending_idles = g_list_prepend (book->priv->pending_idles,
 							    GINT_TO_POINTER (op->idle_id));
@@ -3063,7 +3054,7 @@ fetch_corba_book (EBook       *book,
 	uri = e_source_get_uri (source);
 	if (!uri) {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_OTHER_ERROR,
-			     _("e_book_load_uri: Invalid source."));
+			     _("%s: Invalid source."), "e_book_load_uri");
 		return FALSE;
 	}
 
@@ -3071,7 +3062,7 @@ fetch_corba_book (EBook       *book,
 	factories = activate_factories_for_uri (book, uri);
 	if (!factories) {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_PROTOCOL_NOT_SUPPORTED,
-			     _("e_book_load_uri: no factories available for uri `%s'"), uri);
+			     _("%s: no factories available for uri `%s'"), "e_book_load_uri", uri);
 		return FALSE;
 	}
 
@@ -3083,7 +3074,7 @@ fetch_corba_book (EBook       *book,
 	if (book->priv->listener == NULL) {
 		g_warning ("e_book_load_uri: Could not create EBookListener!\n");
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_OTHER_ERROR,
-			     _("e_book_load_uri: Could not create EBookListener"));
+			     _("%s: Could not create EBookListener"), "e_book_load_uri");
 		return FALSE;
 	}
 	book->priv->listener_signal = g_signal_connect_object (book->priv->listener, "response",
@@ -3135,7 +3126,7 @@ fetch_corba_book (EBook       *book,
 	}
 	else {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_PROTOCOL_NOT_SUPPORTED,
-			     _("e_book_new: no factories available for uri `%s'"), uri);
+			     _("%s: no factories available for uri `%s'"), "e_book_new", uri);
 	}
 		
 	return rv;
@@ -3329,7 +3320,7 @@ e_book_get_self (EContact **contact, EBook **book, GError **error)
 		g_object_unref (*book);
 		*book = NULL;
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_NO_SELF_CONTACT,
-			     _("e_book_get_self: there was no self contact uid stored in gconf"));
+			     _("%s: there was no self contact uid stored in gconf"), "e_book_get_self");
 		return FALSE;
 	}
 
@@ -3440,7 +3431,7 @@ e_book_set_default_source (ESource *source, GError **error)
 	source = e_source_list_peek_source_by_uid (sources, uid);
 	if (!source) {
 		g_set_error (error, E_BOOK_ERROR, E_BOOK_ERROR_NO_SUCH_SOURCE,
-			     _("e_book_set_default_source: there was no source for uid `%s' stored in gconf."), uid);
+			     _("%s: there was no source for uid `%s' stored in gconf."), "e_book_set_default_source", uid);
 		g_object_unref (sources);
 		return FALSE;
 	}
