@@ -49,8 +49,6 @@ compare_im (EContact *contact, const char *str,
 	for (l = aims; l != NULL; l = l->next) {
 		char *im = (char *) l->data;
 
-		g_warning ("Comparing %s and %s", str, im);
-		
 		if (im && compare (im, str))
 			found_it = TRUE;
 
@@ -95,13 +93,20 @@ compare_im_jabber (EContact *contact, const char *str,
 	return compare_im (contact, str, compare, E_CONTACT_IM_JABBER);
 }
 
+static gboolean
+compare_im_groupwise (EContact *contact, const char *str,
+		      char *(*compare)(const char*, const char*))
+{
+	return compare_im (contact, str, compare, E_CONTACT_IM_GROUPWISE);
+}
+
 static gboolean
 compare_email (EContact *contact, const char *str,
 	       char *(*compare)(const char*, const char*))
 {
 	int i;
 
-	for (i = E_CONTACT_EMAIL_1; i <= E_CONTACT_EMAIL_3; i ++) {
+	for (i = E_CONTACT_EMAIL_1; i <= E_CONTACT_EMAIL_4; i ++) {
 		const char *email = e_contact_get_const (contact, i);
 
 		if (email && compare(email, str))
@@ -144,6 +149,14 @@ compare_name (EContact *contact, const char *str,
 	name = e_contact_get_const (contact, E_CONTACT_FAMILY_NAME);
 	if (name && compare (name, str))
 		return TRUE;
+	
+	name = e_contact_get_const (contact, E_CONTACT_GIVEN_NAME);
+	if (name && compare (name, str))
+		return TRUE;
+
+	name = e_contact_get_const (contact, E_CONTACT_NICKNAME);
+	if (name && compare (name, str))
+		return TRUE;
 
 	return FALSE;
 }
@@ -152,8 +165,30 @@ static gboolean
 compare_address (EContact *contact, const char *str,
 		 char *(*compare)(const char*, const char*))
 {
-	g_warning("address searching not implemented\n");
-	return FALSE;
+	
+	int i;
+	gboolean rv = FALSE;
+
+	for (i = E_CONTACT_FIRST_ADDRESS_ID; i <= E_CONTACT_LAST_ADDRESS_ID; i ++) {
+		EContactAddress *address = e_contact_get (contact, i);
+		if (address) {
+			rv =  (address->po && compare(address->po, str)) ||
+				(address->street && compare(address->street, str)) ||
+				(address->ext && compare(address->ext, str)) ||
+				(address->locality && compare(address->locality, str)) ||
+				(address->region && compare(address->region, str)) || 
+				(address->code && compare(address->code, str)) ||
+				(address->country && compare(address->country, str));
+			
+			e_contact_address_free (address);
+		
+			if (rv)
+				break;
+		}
+	}
+
+	return rv;
+	
 }
 
 static gboolean
@@ -215,12 +250,12 @@ static struct prop_info {
 	NORMAL_PROP ( E_CONTACT_NICKNAME, "nickname"),
 	NORMAL_PROP ( E_CONTACT_SPOUSE, "spouse" ),
 	NORMAL_PROP ( E_CONTACT_NOTE, "note"),
-	NORMAL_PROP ( E_CONTACT_UID, "id"),
 	LIST_PROP ( "im_aim",    compare_im_aim ),
 	LIST_PROP ( "im_msn",    compare_im_msn ),
 	LIST_PROP ( "im_icq",    compare_im_icq ),
 	LIST_PROP ( "im_jabber", compare_im_jabber ),
 	LIST_PROP ( "im_yahoo",  compare_im_yahoo ),
+	LIST_PROP ( "im_groupwise", compare_im_groupwise ),
 	LIST_PROP ( "email",     compare_email ),
 	LIST_PROP ( "phone",     compare_phone ),
 	LIST_PROP ( "address",   compare_address ),
