@@ -196,7 +196,9 @@ get_deltas (gpointer handle)
 	char *comp_str;
 	char *time_string = NULL;
 	char t_str [100]; 
+	const char *serv_time;
 	struct stat buf;
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
         
 	if (!handle)
 		return FALSE;
@@ -210,20 +212,25 @@ get_deltas (gpointer handle)
 	if (priv->mode == CAL_MODE_LOCAL)
 		return FALSE;
 
-	g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 100);
-	if (!*t_str || !strcmp (t_str, "")) {
-		icaltimetype temp;
-		time_t current_time;
-		const struct tm *tm;
+	g_static_mutex_lock (&connecting);
 
-		g_warning (" Could not get the correct time stamp for using in getQuick Messages\n");
-		temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
-		current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-		tm = gmtime (&current_time);
-		strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	serv_time = e_cal_backend_cache_get_server_utc_time (cache);
+	if (serv_time) {
+		g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 100);
+		if (!*t_str || !strcmp (t_str, "")) {
+			icaltimetype temp;
+			time_t current_time;
+			const struct tm *tm;
+
+			g_warning (" Could not get the correct time stamp for using in getQuick Messages\n");
+			temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+			current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
+			tm = gmtime (&current_time);
+			strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+		}
+		time_string = g_strdup (t_str);
 	}
 
-	time_string = g_strdup (t_str);
 	status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id, "attachments recipients message recipientStatus default peek", &time_string, "New", "CalendarItem", NULL,  -1,  &item_list);
 	
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
@@ -231,10 +238,13 @@ get_deltas (gpointer handle)
 	
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 				
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 	/* store the timestamp in the cache */	
@@ -282,10 +292,13 @@ get_deltas (gpointer handle)
 		
 	g_free (time_string);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) { 
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 
@@ -328,10 +341,13 @@ get_deltas (gpointer handle)
 		status = e_gw_connection_get_quick_messages (cnc, cbgw->priv->container_id, "iCalId", NULL, "All", "CalendarItem", NULL,  -1,  &item_list);
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
-		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) 
+		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
+			g_static_mutex_unlock (&connecting);
 			return TRUE;
+		}
 
 		e_cal_backend_groupwise_notify_error_code (cbgw, status);
+		g_static_mutex_unlock (&connecting);
 		return TRUE;
 	}
 
@@ -378,10 +394,31 @@ get_deltas (gpointer handle)
 		g_slist_free (cache_keys);
 		item_list = NULL;
 	}
+	
+	g_static_mutex_unlock (&connecting);
 		
         return TRUE;        
 }
 
+static gboolean
+get_deltas_timeout (gpointer cbgw)
+{
+	GThread *thread;
+	GError *error = NULL;
+
+	if (!cbgw)
+		return FALSE;
+
+	thread = g_thread_create ((GThreadFunc) get_deltas, cbgw, FALSE, &error);
+	if (!thread) {
+		g_warning (G_STRLOC ": %s", error->message);
+		g_error_free (error);
+	}
+
+	return TRUE;
+}
+
+
 static char* 
 form_uri (ESource *source)
 {
@@ -468,7 +505,7 @@ cache_init (ECalBackendGroupwise *cbgw)
 
 			/*  Set up deltas only if it is a Calendar backend */
 			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
 			return GNOME_Evolution_Calendar_Success;
 		}
@@ -495,7 +532,7 @@ cache_init (ECalBackendGroupwise *cbgw)
 		/* get the deltas from the cache */
 		if (get_deltas (cbgw)) {
 			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
 			return GNOME_Evolution_Calendar_Success;
 		} else {
@@ -559,7 +596,7 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 
 					return GNOME_Evolution_Calendar_OtherError;
 				}
-				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas, (gpointer) cbgw);
+				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas_timeout, (gpointer)cbgw);
 
 			}
 	
@@ -1135,6 +1172,7 @@ e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal
 		}
         }
 
+	g_object_unref (cbsexp);
 	g_list_foreach (components, (GFunc) g_object_unref, NULL);
 	g_list_free (components);
 
