@@ -3,7 +3,7 @@
   FILE: icaltime.c
   CREATOR: eric 02 June 2000
   
-  $Id: icaltime.c,v 1.34 2004/12/22 11:54:39 rodrigo Exp $
+  $Id: icaltime.c,v 1.34.4.1 2005/03/15 01:33:54 jpr Exp $
   $Locker:  $
     
  (C) COPYRIGHT 2000, Eric Busboom, http://www.softwarestudio.org
@@ -347,8 +347,8 @@ void unset_tz(char *tzstr)
     } 
 
     /* Free any previous TZ environment string we have used. */
-    //if (saved_tz)
-    //  free (saved_tz);
+    if (saved_tz)
+	free (saved_tz);
 
     /* Save a pointer to the TZ string we just set, so we can free it later.
        (This can possibly be NULL if there was no TZ to restore.) */
