@@ -1233,7 +1233,7 @@ rfc2047_encode_word(GString *outstring, const char *in, size_t len, const char *
 			if (e_iconv (ic, &inptr, &convlen, &out, &outlen) == (size_t) -1 && errno != EINVAL) {
 				w(g_warning("Conversion problem: conversion truncated: %s", strerror (errno)));
 				/* blah, we include it anyway, better than infinite loop ... */
-				inptr = p + convlen;
+				inptr += convlen;
 			} else {
 				/* make sure we flush out any shift state */
 				e_iconv (ic, NULL, 0, &out, &outlen);
@@ -1272,6 +1272,7 @@ camel_header_encode_string (const unsigned char *in)
 	const unsigned char *inptr = in, *start, *word;
 	gboolean last_was_encoded = FALSE;
 	gboolean last_was_space = FALSE;
+	const char *charset;
 	int encoding;
 	GString *out;
 	char *outstr;
@@ -1335,8 +1336,9 @@ camel_header_encode_string (const unsigned char *in)
 				if (last_was_encoded)
 					g_string_append_c (out, ' ');
 				
-				rfc2047_encode_word (out, start, inptr - start,
-						     camel_charset_best (start, inptr - start), CAMEL_MIME_IS_ESAFE);
+				if (!(charset = camel_charset_best (start, inptr - start)))
+					charset = "UTF-8";
+				rfc2047_encode_word (out, start, inptr - start, charset, CAMEL_MIME_IS_ESAFE);
 				last_was_encoded = TRUE;
 				break;
 			}
@@ -1381,8 +1383,9 @@ camel_header_encode_string (const unsigned char *in)
 			if (last_was_encoded)
 				g_string_append_c (out, ' ');
 			
-			rfc2047_encode_word (out, start, inptr - start,
-					     camel_charset_best (start, inptr - start - 1), CAMEL_MIME_IS_ESAFE);
+			if (!(charset = camel_charset_best (start, inptr - start)))
+				charset = "UTF-8";
+			rfc2047_encode_word (out, start, inptr - start, charset, CAMEL_MIME_IS_ESAFE);
 			break;
 		}
 	}
@@ -1536,7 +1539,7 @@ header_encode_phrase_merge_words (GList **wordsp)
 				if (MERGED_WORD_LT_FOLDLEN (next->end - word->start, MAX (word->type, next->type))) {
 					/* the resulting word type is the MAX of the 2 types */
 					word->type = MAX(word->type, next->type);
-					
+					word->encoding = MAX(word->encoding, next->encoding);
 					word->end = next->end;
 					words = g_list_remove_link (words, nextl);
 					g_list_free_1 (nextl);
@@ -1570,6 +1573,7 @@ camel_header_encode_phrase (const unsigned char *in)
 {
 	struct _phrase_word *word = NULL, *last_word = NULL;
 	GList *words, *wordl;
+	const char *charset;
 	GString *out;
 	char *outstr;
 	
@@ -1621,11 +1625,13 @@ camel_header_encode_phrase (const unsigned char *in)
 				start = word->start;
 			}
 			
-			if (word->encoding == 1)
+			if (word->encoding == 1) {
 				rfc2047_encode_word (out, start, len, "ISO-8859-1", CAMEL_MIME_IS_PSAFE);
-			else
-				rfc2047_encode_word (out, start, len,
-						     camel_charset_best (start, len), CAMEL_MIME_IS_PSAFE);
+			} else {
+				if (!(charset = camel_charset_best (start, len)))
+					charset = "UTF-8";
+				rfc2047_encode_word (out, start, len, charset, CAMEL_MIME_IS_PSAFE);
+			}
 			break;
 		}
 		
@@ -2102,6 +2108,9 @@ header_decode_addrspec(const char **in)
 		}
 	} else {
 		w(g_warning("invalid addr-spec, no local part"));
+		g_string_free(addr, TRUE);
+
+		return NULL;
 	}
 
 	/* FIXME: return null on error? */
@@ -2808,9 +2817,9 @@ header_append_param(struct _camel_header_param *last, char *name, char *value)
 	if (strncmp(value, "=?", 2) == 0
 	    && (node->value = header_decode_text(value, strlen(value), FALSE, NULL))) {
 		g_free(value);
-	} else if (!g_utf8_validate(value, -1, NULL)) {
-		const char * charset = e_iconv_locale_charset();
-
+	} else if (g_ascii_strcasecmp (name, "boundary") != 0 && !g_utf8_validate(value, -1, NULL)) {
+		const char *charset = e_iconv_locale_charset();
+		
 		if ((node->value = header_convert("UTF-8", charset?charset:"ISO-8859-1", value, strlen(value)))) {
 			g_free(value);
 		} else {
