@@ -35,14 +35,23 @@
 #include <libebook/e-contact.h>
 #include <libedataserverui/e-contact-store.h>
 #include <libedataserverui/e-destination-store.h>
+#include <libedataserverui/e-book-auth-util.h>
 #include "e-name-selector.h"
 
+#define PRIVATE_SOURCE_BOOKS_KEY "private-source-books"
+
 typedef struct {
 	gchar              *name;
 	ENameSelectorEntry *entry;
 }
 Section;
 
+typedef struct {
+	EBook *book;
+	guint  is_completion_book : 1;
+}
+SourceBook;
+
 static void  e_name_selector_init       (ENameSelector		 *name_selector);
 static void  e_name_selector_class_init (ENameSelectorClass	 *name_selector_class);
 static void  e_name_selector_finalize   (GObject                 *object);
@@ -53,6 +62,20 @@ static void  e_name_selector_finalize   (GObject                 *object);
 
 G_DEFINE_TYPE (ENameSelector, e_name_selector, G_TYPE_OBJECT);
 
+static void
+source_books_destroy (GArray *source_books)
+{
+	gint i;
+
+	for (i = 0; i < source_books->len; i++) {
+		SourceBook *source_book = &g_array_index (source_books, SourceBook, i);
+
+		g_object_unref (source_book->book);
+	}
+
+	g_array_free (source_books, TRUE);
+}
+
 static void
 e_name_selector_class_init (ENameSelectorClass *name_selector_class)
 {
@@ -66,8 +89,57 @@ e_name_selector_class_init (ENameSelectorClass *name_selector_class)
 static void
 e_name_selector_init (ENameSelector *name_selector)
 {
+	ESourceList *source_list;
+	GArray      *source_books;
+	GSList      *groups;
+	GSList      *l;
+
 	name_selector->sections = g_array_new (FALSE, FALSE, sizeof (Section));
 	name_selector->model = e_name_selector_model_new ();
+
+	/* Make a list of books */
+
+	source_books = g_array_new (FALSE, FALSE, sizeof (SourceBook));
+
+	/* This should be a private field, but we use g_object_set_data() to maintain
+	 * ABI compatibility in the GNOME 2.10 branch */
+	g_object_set_data_full (G_OBJECT (name_selector), PRIVATE_SOURCE_BOOKS_KEY, source_books,
+				(GDestroyNotify) source_books_destroy);
+
+	if (!e_book_get_addressbooks (&source_list, NULL)) {
+		g_warning ("ENameSelector can't find any addressbooks!");
+		return;
+	}
+
+	groups = e_source_list_peek_groups (source_list);
+
+	for (l = groups; l; l = g_slist_next (l)) {
+		ESourceGroup *group   = l->data;
+		GSList       *sources = e_source_group_peek_sources (group);
+		GSList       *m;
+
+		for (m = sources; m; m = g_slist_next (m)) {
+			ESource      *source = m->data;
+			const gchar  *completion;
+			SourceBook    source_book;
+
+			/* We're only loading completion books for now, as we don't want
+			 * unnecessary auth prompts */
+			completion = e_source_get_property (source, "completion");
+			if (!completion || g_ascii_strcasecmp (completion, "true"))
+				continue;
+
+			source_book.book = e_load_book_source (source, NULL, NULL);
+			if (!source_book.book)
+				continue;
+
+			source_book.is_completion_book = TRUE;
+
+			g_array_append_val (source_books, source_book);
+		}
+	}
+
+	g_object_unref (source_list);
 }
 
 static void
@@ -183,14 +255,33 @@ e_name_selector_peek_section_entry (ENameSelector *name_selector, const gchar *n
 	section = &g_array_index (name_selector->sections, Section, n);
 
 	if (!section->entry) {
-		gchar *text;
+		GArray        *source_books;
+		EContactStore *contact_store;
+		gchar         *text;
+		gint           i;
+
 		section->entry = e_name_selector_entry_new ();
 		if (pango_parse_markup (name, -1, '_', NULL,
-				&text, NULL, NULL))  {
+					&text, NULL, NULL))  {
 			atk_object_set_name (gtk_widget_get_accessible (GTK_WIDGET (section->entry)), text);
 			g_free (text);
 		}
 		e_name_selector_entry_set_destination_store (section->entry, destination_store);
+
+		/* Create a contact store for the entry and assign our already-open books to it */
+
+		contact_store = e_contact_store_new ();
+		source_books = g_object_get_data (G_OBJECT (name_selector), PRIVATE_SOURCE_BOOKS_KEY);
+
+		for (i = 0; i < source_books->len; i++) {
+			SourceBook *source_book = &g_array_index (source_books, SourceBook, i);
+
+			if (source_book->is_completion_book)
+				e_contact_store_add_book (contact_store, source_book->book);
+		}
+
+		e_name_selector_entry_set_contact_store (section->entry, contact_store);
+		g_object_unref (contact_store);
 	}
 
 	return section->entry;
